from scipy._typing import Untyped
from scipy.fft._pocketfft import (
    fft as fft,
    fftn as fftn,
    hfft as hfft,
    hfftn as hfftn,
    ifft as ifft,
    ifftn as ifftn,
    ihfft as ihfft,
    ihfftn as ihfftn,
    irfft as irfft,
    irfftn as irfftn,
    rfft as rfft,
    rfftn as rfftn,
)

LARGE_COMPOSITE_SIZES: Untyped
SMALL_COMPOSITE_SIZES: Untyped
LARGE_PRIME_SIZES: Untyped
SMALL_PRIME_SIZES: Untyped

def random(size) -> Untyped: ...
def swap_byteorder(arr) -> Untyped: ...
def direct_dft(x) -> Untyped: ...
def direct_idft(x) -> Untyped: ...
def direct_dftn(x) -> Untyped: ...
def direct_idftn(x) -> Untyped: ...
def direct_rdft(x) -> Untyped: ...
def direct_irdft(x, n) -> Untyped: ...
def direct_rdftn(x) -> Untyped: ...

class _TestFFTBase:
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...
    def test_definition(self): ...
    def test_n_argument_real(self): ...
    def test_djbfft(self): ...
    def test_invalid_sizes(self): ...

class TestLongDoubleFFT(_TestFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class TestDoubleFFT(_TestFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class TestSingleFFT(_TestFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class TestFloat16FFT:
    def test_1_argument_real(self): ...
    def test_n_argument_real(self): ...

class _TestIFFTBase:
    def setup_method(self): ...
    def test_definition(self): ...
    def test_definition_real(self): ...
    def test_djbfft(self): ...
    def test_random_complex(self): ...
    def test_random_real(self): ...
    def test_size_accuracy(self): ...
    def test_invalid_sizes(self): ...

class TestLongDoubleIFFT(_TestIFFTBase):
    cdt: Untyped
    rdt: Untyped
    rtol: float
    atol: float
    def setup_method(self): ...

class TestDoubleIFFT(_TestIFFTBase):
    cdt: Untyped
    rdt: Untyped
    rtol: float
    atol: float
    def setup_method(self): ...

class TestSingleIFFT(_TestIFFTBase):
    cdt: Untyped
    rdt: Untyped
    rtol: float
    atol: float
    def setup_method(self): ...

class _TestRFFTBase:
    def setup_method(self): ...
    def test_definition(self): ...
    def test_djbfft(self): ...
    def test_invalid_sizes(self): ...
    def test_complex_input(self): ...
    class MockSeries:
        data: Untyped
        def __init__(self, data) -> None: ...
        def __getattr__(self, item) -> Untyped: ...

    def test_non_ndarray_with_dtype(self): ...

class TestRFFTLongDouble(_TestRFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class TestRFFTDouble(_TestRFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class TestRFFTSingle(_TestRFFTBase):
    cdt: Untyped
    rdt: Untyped
    def setup_method(self): ...

class _TestIRFFTBase:
    def setup_method(self): ...
    def test_definition(self): ...
    def test_djbfft(self): ...
    def test_random_real(self): ...
    def test_size_accuracy(self): ...
    def test_invalid_sizes(self): ...

class TestIRFFTLongDouble(_TestIRFFTBase):
    cdt: Untyped
    rdt: Untyped
    ndec: int
    def setup_method(self): ...

class TestIRFFTDouble(_TestIRFFTBase):
    cdt: Untyped
    rdt: Untyped
    ndec: int
    def setup_method(self): ...

class TestIRFFTSingle(_TestIRFFTBase):
    cdt: Untyped
    rdt: Untyped
    ndec: int
    def setup_method(self): ...

class TestFftnSingle:
    def setup_method(self): ...
    def test_definition(self): ...
    def test_size_accuracy_small(self, size): ...
    def test_size_accuracy_large(self, size): ...
    def test_definition_float16(self): ...
    def test_float16_input_small(self, size): ...
    def test_float16_input_large(self, size): ...

class TestFftn:
    def setup_method(self): ...
    def test_definition(self): ...
    def test_axes_argument(self): ...
    def test_shape_argument(self): ...
    def test_shape_axes_argument(self): ...
    def test_shape_axes_argument2(self): ...
    def test_shape_argument_more(self): ...
    def test_invalid_sizes(self): ...
    def test_no_axes(self): ...
    def test_regression_244(self): ...

class TestIfftn:
    dtype: Untyped
    cdtype: Untyped
    def setup_method(self): ...
    def test_definition(self, dtype, cdtype, maxnlp): ...
    def test_random_complex(self, maxnlp, size): ...
    def test_invalid_sizes(self): ...
    def test_no_axes(self): ...

class TestRfftn:
    dtype: Untyped
    cdtype: Untyped
    def setup_method(self): ...
    def test_definition(self, dtype, cdtype, maxnlp): ...
    def test_random(self, size): ...
    def test_invalid_sizes(self, func): ...
    def test_no_axes(self, func): ...
    def test_complex_input(self): ...

class FakeArray:
    __array_interface__: Untyped
    def __init__(self, data) -> None: ...

class FakeArray2:
    def __init__(self, data) -> None: ...
    def __array__(self, dtype: Untyped | None = None, copy: Untyped | None = None) -> Untyped: ...

class TestOverwrite:
    real_dtypes: Untyped
    dtypes: Untyped
    fftsizes: Untyped
    def test_fft_ifft(self, dtype, fftsize, overwrite_x, shape, axes): ...
    def test_rfft_irfft(self, dtype, fftsize, overwrite_x, shape, axes): ...
    def test_fftn_ifftn(self, dtype, overwrite_x, shape, axes): ...

def test_invalid_norm(func): ...
def test_swapped_byte_order_complex(func): ...
def test_swapped_byte_order_real(func): ...
